% Particle smoother according to 
%
% A. Doucet, S.J. Godsill and C. Andrieu, On Sequential Monte Carlo 
% sampling methods for Bayesian filtering, Statistics and Computing,
% 10(3):197-208, 2000.
%

function g = psDGA(m,M,z,gPF)
  N       = size(z.y,2);    % Number of samples
  ws      = zeros(N,M);     % Initialize the smoothed weights
  ws(N,:) = gPF.w(N,:);     % 2. Initialize the smoothed weights
  wij     = zeros(N-1,M,M); % Initialize the weights for I2 (50)
  dynNorm = 1/(sqrt(2*pi)*sqrt(m.ss.Q));
  for t = N-1:-1:1
    for i = 1:M
      v = zeros(1,M);
      for k = 1:M
        dyn  = dynNorm*exp(-0.5*(repmat(gPF.xPWeighted(t+1,k),1,M) - m.ss.A*gPF.xPWeighted(t,:)).^2/m.ss.Q);
        v(k) = sum(gPF.w(t,:).*dyn);    % (46c)
      end;
      dyn     = dynNorm*exp(-0.5*(gPF.xPWeighted(t+1,:) - m.ss.A*repmat(gPF.xPWeighted(t,i),1,M)).^2/m.ss.Q);
      dynv    = dyn./v;
      wtmp    = sum(ws(t+1,:).*dynv);
      ws(t,i) = gPF.w(t,i)*wtmp;       % (46b)
    end;
  end;
  % Compute the weight matrix for approximating I2 (50)
  for t = 1:N-1
    for i = 1:M
      for j = 1:M
       dyn        = dynNorm*exp(-0.5*(repmat(gPF.xPWeighted(t+1,j),1,M) - m.ss.A*gPF.xPWeighted(t,:)).^2/m.ss.Q);
       dynDen     = sum(gPF.w(t,:).*dyn);
       dynNum     = dynNorm*exp(-0.5*(gPF.xPWeighted(t+1,j) - m.ss.A*gPF.xPWeighted(t,i))^2/m.ss.Q);
       wij(t,i,j) = gPF.w(t,i)*ws(t+1,j)*dynNum/dynDen;
      end;
    end;
  end;
    
  xs = zeros(1,N);
  for t=1:N   % Compute the smoothed state estimates
    xs(t) = sum(ws(t,:).*gPF.xPWeighted(t,:));
  end;
  g.ws    = ws;              % Return the smoothed weights
  g.xPart = gPF.xPWeighted;  % Return the particles
  g.Xs    = xs;              % Smoothed states estimate
  g.wij   = wij;             % Weights for I2 (50)
end
