addpath('../UNIT');    % Add path to UNIT toolbox
plotSmoother2000 = 0;
plotSmoother2010 = 1;
plotTrue         = 1;

%==========================================================
%===   Simulate the one dimensional state-space model   ===
%==========================================================
N = 100;          % Number of data
m.nx = 1;
m.ss.A = 0.9;     m.ss.B = 0;
m.ss.C = 0.5;     m.ss.D = 0;
m.ss.Q = 0.1;     m.ss.S = 0;
m.ss.R = 0.1;

m.ss.X1 = 0;      m.ss.P1 = 0;  % Initial state and covariance
x = zeros(1,N+1); 
y = zeros(1,N);
x(1) = m.ss.X1;
v = sqrt(m.ss.Q)*randn(1,N);    % Process noise sequence
e = sqrt(m.ss.R)*randn(1,N);    % Measurement noise sequence
for t=1:N
  x(t+1) = m.ss.A*x(t) + v(t);
  y(t)   = m.ss.C*x(t) + e(t);
end
z.y = y;

gRTS = ks(z,m);          % Linear optimal smoother

M = 1000;                % Number of particles

disp('Running the particle filter.')
tic
gPF = pf(m,M,z);         % Particle filter
toc
if plotSmoother2000
	disp('Running the particle smoother (2000).')
	tic
	gPSDGA = psDGA(m,M,z,gPF);  % Particle smoother according to Doucet, Godsill, and Andrieu (2000)
	toc
end
if plotSmoother2010
	disp('Running the particle smoother (2010).')
	tic
	gPS = ps(m,M,gPF);       % Particle smoother based on Douc, Garivier, Moulines and Olsson (2010)
	toc
end

if plotSmoother2000
  figure(1)
  plot(gRTS.ss.Xs(1,1:N),'r','LineWidth',1.5)
  hold on;
  plot(gPSDGA.Xs(1,1:N),'b')
  if plotTrue
    plot(x(1,1:N),'k')
  end;
  hold off;
  xlabel('Time')
  ylabel('x')
  title('RTS smoother - red, Particle smoother (2000) - blue')  
end;
if plotSmoother2010
  figure(2)
  plot(gRTS.ss.Xs(1,1:N),'r','LineWidth',1.5)
  hold on;
  plot(gPS.Xs(1,1:N),'b')
  if plotTrue
    plot(x(1,1:N),'k')
  end;
  hold off;
  xlabel('Time')
  ylabel('x')
  title('RTS smoother - red, Particle smoother (2010) - blue')
end;
