% Particle filter

function g = pf(m,M,z)
  y           = z.y;
  N           = size(y,2);       % Number of samples
  xf          = zeros(1,size(y,2));
  xPWeighted  = zeros(N,M);      % Make some room
  xPResampled = zeros(N,M);      % Make some room
  wStore      = zeros(N,M);      % Make some room

  % 1. Initialize particles
  if m.P1==0
    x = repmat(m.X1,1,M);    % Initial state fully known
  else
    x = repmat(m.X1,1,M) + sqrt(m.P1)*randn(1,M);
  end;
  for t=1:N
    e = repmat(y(t),1,M) - m.d*x.^2;
    w = exp(-(1/2)*(e.*(m.R\e)));  % 2. Evaluate the likelihood
    w = w/sum(w);
    xPWeighted(t,:) = x;
    wStore(t,:)     = w;
    xf(t) = sum(w.*x);             % Compute state estimate
    index = sysresample(w);        % 3. Resample
    x     = x(index);
    xPResampled(t,:) = x;
    x = m.a*x + m.b*(x./(1+x.^2)) + m.c*cos(1.2*t) + sqrt(m.Q)*randn(1,M); % 4. Predict the particles
  end;
  g.Xf          = xf;
  g.xPWeighted  = xPWeighted;
  g.xPResampled = xPResampled;
  g.w           = wStore;
end
